;/*--------------------------------------------------------------------------------------------------------*/
;/* Winbond Electronics Corporation confidential                                                           */
;/*                                                                                                        */
;/* Copyright (c) 2007 by Winbond Electronics Corporation                                                  */
;/* All rights reserved                                                                                    */
;/*                                                                                                        */
;/*--------------------------------------------------------------------------------------------------------*/
;/* File Contents:                                                                                         */
;/*   wb_init.s                                                                                            */
;/*          The init assembly.                                                                            */
;/*                                                                                                        */
;/* Version:                                                                                               */
;/*          v0.0.1                                                                                        */
;/*                                                                                                        */
;/* Project:                                                                                               */
;/*          DA8205                                                                                        */
;/*                                                                                                        */
;/*                                                                               $Date: 07/10/07 4:20p $  */
;/*--------------------------------------------------------------------------------------------------------*/

    EXPORT      Reset_Go
    EXPORT      _sys_exit

    IMPORT      ||Load$$RAM$$Base||
    IMPORT      ||Image$$RAM$$Base||
    IMPORT      ||Image$$RAM$$Length||
    IMPORT      ||Image$$RAM$$ZI$$Base||
    IMPORT      ||Image$$RAM$$ZI$$Limit||

    IMPORT      __main


 	PRESERVE8
    AREA    app_init, CODE, READONLY

;/*--------------------------------------------------------------------------------------------------------*/
;/* Standard definitions of mode bits and interrupt (I & F) flags in PSRs                                  */
;/*--------------------------------------------------------------------------------------------------------*/
Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UNDEF      EQU     0x1B
Mode_SYS        EQU     0x1F                ; available on ARM Arch 4 and later

I_Bit           EQU     0x80                ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40                ; when F bit is set, FIQ is disabled

;/*--------------------------------------------------------------------------------------------------------*/
;/* System memory locations                                                                                */
;/*--------------------------------------------------------------------------------------------------------*/

RAM_Limit       EQU     0x00008000


UND_Stack_Len   EQU     8                  
Abort_Stack_Len EQU     8                  
IRQ_Stack_Len   EQU     256                 
FIQ_Stack_Len   EQU     8                  
SVC_Stack_Len   EQU     128                 


UND_Stack       EQU     RAM_Limit
Abort_Stack     EQU     UND_Stack   - UND_Stack_Len
IRQ_Stack       EQU     Abort_Stack - Abort_Stack_Len
FIQ_Stack       EQU     IRQ_Stack   - IRQ_Stack_Len
SVC_Stack       EQU     FIQ_Stack   - FIQ_Stack_Len
USR_Stack       EQU     SVC_Stack   - SVC_Stack_Len




    ENTRY

;/*--------------------------------------------------------------------------------------------------------*/
;/* Exception Vectors                                                                                      */
;/*--------------------------------------------------------------------------------------------------------*/


Vector_Start
    B       InitDMM
    LDR     PC, Undefined_Addr
    LDR     PC, SWI_Addr
    LDR     PC, Prefetch_Addr
    LDR     PC, Abort_Addr
    NOP                                     ; Reserved vector
    LDR     PC, IRQ_Addr
    LDR     PC, FIQ_Addr

Reset_Addr      DCD     Reset_Go
Undefined_Addr  DCD     UndefinedHandler
SWI_Addr        DCD     SwiHandler
Prefetch_Addr   DCD     PrefetchAbortHandler
Abort_Addr      DCD     DataAbortHandler
                DCD     0                   ; Reserved vector
IRQ_Addr        DCD     DefaultISR
FIQ_Addr        DCD     FIQHandler

UndefinedHandler
		b	UndefinedHandler

SwiHandler
		b	SwiHandler

PrefetchAbortHandler
		b	PrefetchAbortHandler

DataAbortHandler
		b	DataAbortHandler

DefaultISR
		b	DefaultISR

FIQHandler
		b	FIQHandler

Vector_Limit




;/*--------------------------------------------------------------------------------------------------------*/
;/* Init Direct Memory Mode                                                                                */
;/*--------------------------------------------------------------------------------------------------------*/
InitDMM
		LDR r0, =0xb1007008	  ; Auto chip select
		LDR r1, =0x00000009	  ; 
		STR r1, [r0, #0x00]

		LDR r0, =0xb1007000	  ; DMM mode
		LDR r1, =0x0B321344	  ; 
		STR r1, [r0, #0x00]

		LDR r0, =0xb1007004	  ; Divider
		LDR r1, =0x007F0000	  ; 
		STR r1, [r0, #0x00]

    	LDR r0,=0xb1000034
    	LDR r1,=0x140
    	STR r1, [r0, #0x00]

        
Reset_Go

 
    ;/*----------------------------------------------------------------------------------------------------*/
    ;/* Initialise stack pointer registers                                                                 */
    ;/*----------------------------------------------------------------------------------------------------*/
    MSR     CPSR_c, #Mode_UNDEF:OR:I_Bit:OR:F_Bit
    LDR     SP, =UND_Stack

    MSR     CPSR_c, #Mode_ABT:OR:I_Bit:OR:F_Bit
    LDR     SP, =Abort_Stack

    MSR     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit
    LDR     SP, =IRQ_Stack

    MSR     CPSR_c, #Mode_FIQ:OR:I_Bit:OR:F_Bit
    LDR     SP, =FIQ_Stack

    MSR     CPSR_c, #Mode_SYS:OR:I_Bit:OR:F_Bit
    LDR     SP, =USR_Stack

    MSR     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit
    LDR     SP, =SVC_Stack



    ;/*----------------------------------------------------------------------------------------------------*/
    ;/* Now enter the C code                                                                               */
    ;/*----------------------------------------------------------------------------------------------------*/
    LDR     r0, =__main
    CMP     r0, #0
    MOVNE   lr, pc
    BXNE    r0                          

    ;/*----------------------------------------------------------------------------------------------------*/
    ;/* If no C code, just try to exit by semihosed swi                                                    */
    ;/* Try to install our SWI handler to avoid semihosted program crash when no host existed              */
    ;/*----------------------------------------------------------------------------------------------------*/
    LDR     r0, =0xE1B0F00E                 ; MOVS    pc, lr (return from SWI)
    MOV     r1, #0x08
    STR     r0, [r1]

    ;/*----------------------------------------------------------------------------------------------------*/
    ;/* Stop by SWI or forever loop                                                                        */
    ;/*----------------------------------------------------------------------------------------------------*/
    MOV     r0, #0x18
    SWI     0x123456
_sys_exit
99  B       %B99                            ; forever loop


	


    LTORG

exit

    END
