/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include "Driver/DrvGPIO.h" 
#include "System/SysInfra.h"
#include "Driver/DrvSYS.h"	
#include "Driver/DrvUART.h"
#include "Driver/DrvSerialIO.h"
#include "Driver/DrvAIC.h"


VOID GPIO_KeyHandler(UINT32 u32UserData)
{
    UINT32 u32IoStatus, u32TriggerType;
    INT32 i;
    volatile PUINT32 pu32Reg;
    
    pu32Reg = DrvGPIO_GetIntStatus();
    
    /* Get the interrupt status */
    u32IoStatus = pu32Reg[0];
    
    /* Handle the interrupt */
    for(i=0;i<4;i++)
    {
        if(u32IoStatus & (1 << (i + 3)))
        {
            /* Clean the interrupt (Write 1 clear) */
            pu32Reg[0] = (1 << (i + 3));
        
            DrvSIO_printf("#%d KEY %c ",u32UserData, 'A' + i);
            
            u32TriggerType = (inp32(GPIOA_PIN) >> (i + 3)) & 1;
            if(u32TriggerType == 1)
            {
                DrvSIO_printf("Released!\n");
                if(DrvGPIO_GetBit(GPIOB, 6))
                    DrvGPIO_ClrBit(GPIOB, 6);
                else
                    DrvGPIO_SetBit(GPIOB, 6);
            }
            else
            {
                DrvSIO_printf("Pressed\n");
                if(DrvGPIO_GetBit(GPIOB, 7))
                    DrvGPIO_ClrBit(GPIOB, 7);
                else
                    DrvGPIO_SetBit(GPIOB, 7);
            }
        }
    }    
}

INT32 main(VOID)
{
    UART_T uart;
    
    
    DrvSYS_Open(12000, 144000);
    
    DrvSYS_SetCPUClock(72000);
    
    uart.u32BaudRate = 115200;
    uart.u8cDataBits = DRVUART_DATABITS_8;
    uart.u8cStopBits = DRVUART_STOPBITS_1;
    uart.u8cParity = DRVUART_PARITY_NONE;
    uart.u8cRxTriggerLevel = DRVUART_FIFO_62BYTES;
    uart.u8TimeOut = 10;
    DrvUART_Open(UART_PORT0, &uart);

	DrvGPIO_Init();
	
	DrvGPIO_InitFunction(FUNC_UART0, 0);
	
	DrvGPIO_InitBitFunction(GPIOB, DRVGPIO_BIT6, FUNC_GPIO);
	DrvGPIO_InitBitFunction(GPIOB, DRVGPIO_BIT7, FUNC_GPIO);
	
	
    DrvGPIO_InitBitFunction(GPIOA, DRVGPIO_BIT3, FUNC_GPIO);
    DrvGPIO_InitBitFunction(GPIOA, DRVGPIO_BIT4, FUNC_GPIO);
    DrvGPIO_InitBitFunction(GPIOA, DRVGPIO_BIT5, FUNC_GPIO);
    DrvGPIO_InitBitFunction(GPIOA, DRVGPIO_BIT6, FUNC_GPIO);
	
	
	DrvGPIO_Open(GPIOB, DRVGPIO_BIT6, IO_OUTPUT, IO_NO_PULL_UP, IO_LOWDRV);
	DrvGPIO_Open(GPIOB, DRVGPIO_BIT7, IO_OUTPUT, IO_NO_PULL_UP, IO_LOWDRV);
	
	
	DrvGPIO_Open(GPIOA, DRVGPIO_BIT3, IO_INPUT, IO_PULL_UP, IO_LOWDRV);
	DrvGPIO_Open(GPIOA, DRVGPIO_BIT4, IO_INPUT, IO_PULL_UP, IO_LOWDRV);
	DrvGPIO_Open(GPIOA, DRVGPIO_BIT5, IO_INPUT, IO_PULL_UP, IO_LOWDRV);
	DrvGPIO_Open(GPIOA, DRVGPIO_BIT6, IO_INPUT, IO_PULL_UP, IO_LOWDRV);
	
	
	DrvGPIO_EnableIntDebounce(IO_INT0, 7);
	DrvGPIO_EnableIntDebounce(IO_INT1, 7);
	DrvGPIO_EnableIntDebounce(IO_INT2, 7);
	DrvGPIO_EnableIntDebounce(IO_INT3, 7);
	
	DrvGPIO_EnableInt(GPIOA, 3, IO_INT0, IO_BOTH_EDGE, (DRVGPIO_CALLBACK)GPIO_KeyHandler, 3);
	DrvGPIO_EnableInt(GPIOA, 4, IO_INT1, IO_BOTH_EDGE, (DRVGPIO_CALLBACK)GPIO_KeyHandler, 4);
	DrvGPIO_EnableInt(GPIOA, 5, IO_INT2, IO_BOTH_EDGE, (DRVGPIO_CALLBACK)GPIO_KeyHandler, 5);
	DrvGPIO_EnableInt(GPIOA, 6, IO_INT3, IO_BOTH_EDGE, (DRVGPIO_CALLBACK)GPIO_KeyHandler, 6);
	
	DrvAIC_SetCPSR(AIC_ENABLE_IRQ);
	
	DrvSIO_printf("+**********************************************************+\n");
    DrvSIO_printf("|                      GPIO Sample Code                    |\n");	
	DrvSIO_printf("+**********************************************************+\n");
	DrvSIO_printf("Current CPU Speed: %d MHz\n", DrvSYS_GetCPUClock() / 1000);
	DrvSIO_printf("GPIOA 3 ==> Key A, GPIOA 4 ==> Key B, GPIOA 5 ==> Key C, GPIOA 6 ==> Key D\n");
	DrvSIO_printf("GPIOB 5 ==> LED5, GPIOB 6 ==> LED6\n");
	

    

    while(1);
	
}







